<?php

declare(strict_types=1);

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 */

use ILIAS\Setup;

/**
 * Store information about https is enabled
 */
class ilPrivacySecurityConfigStoredObjective implements Setup\Objective
{
    /**
     * @var    ilPrivacySecuritySetupConfig
     */
    protected ilPrivacySecuritySetupConfig $config;

    public function __construct(ilPrivacySecuritySetupConfig $config)
    {
        $this->config = $config;
    }

    public function getHash(): string
    {
        return hash("sha256", self::class);
    }

    public function getLabel(): string
    {
        return "Store information about privacy security in settings";
    }

    public function isNotable(): bool
    {
        return true;
    }

    public function getPreconditions(Setup\Environment $environment): array
    {
        return [
            new ilIniFilesPopulatedObjective(),
            new ilSettingsFactoryExistsObjective()
        ];
    }

    public function achieve(Setup\Environment $environment): Setup\Environment
    {
        $factory = $environment->getResource(Setup\Environment::RESOURCE_SETTINGS_FACTORY);
        $settings = $factory->settingsFor("common");
        $settings->set("https", $this->bool2string($this->config->getForceHttpsOnLogin()));

        /** @var $settings ilSetting */
        if (null !== $this->config->getAuthDurationInMs()) {
            $settings->set("auth_duration", (string) $this->config->getAuthDurationInMs());
        } else {
            $settings->delete("auth_duration");
        }

        if (null !== $this->config->getAccountAssistanceDurationInMs()) {
            $settings->set("account_assistance_duration", (string) $this->config->getAccountAssistanceDurationInMs());
        } else {
            $settings->delete("account_assistance_duration");
        }

        return $environment;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable(Setup\Environment $environment): bool
    {
        return true;
    }

    protected function bool2string(bool $value): string
    {
        if ($value) {
            return "1";
        }
        return "0";
    }
}
